'use strict';

/**
 * Module dependencies
 */
import it from '../../it';
import Signin from '../../models/signin';
import serialize from '../../serializers/signin';

/**
 * Get signin history of my account
 *
 * @param {any} params
 * @param {any} user
 * @return {Promise<any>}
 */
module.exports = (params, user) =>
	new Promise(async (res, rej) =>
{
	// Get 'limit' parameter
	const [limit, limitErr] = it(params.limit).expect.number().range(1, 100).default(10).qed();
	if (limitErr) return rej('invalid limit param');

	// Get 'since_id' parameter
	const [sinceId, sinceIdErr] = it(params.since_id).expect.id().qed();
	if (sinceIdErr) return rej('invalid since_id param');

	// Get 'max_id' parameter
	const [maxId, maxIdErr] = it(params.max_id).expect.id().qed();
	if (maxIdErr) return rej('invalid max_id param');

	// Check if both of since_id and max_id is specified
	if (sinceId !== null && maxId !== null) {
		return rej('cannot set since_id and max_id');
	}

	const query = {
		user_id: user._id
	} as any;

	const sort = {
		_id: -1
	};

	if (sinceId) {
		sort._id = 1;
		query._id = {
			$gt: sinceId
		};
	} else if (maxId) {
		query._id = {
			$lt: maxId
		};
	}

	// Issue query
	const history = await Signin
		.find(query, {
			limit: limit,
			sort: sort
		});

	// Serialize
	res(await Promise.all(history.map(async record =>
		await serialize(record))));
});
