'use strict';

/**
 * Module dependencies
 */
import it from '../it';
import User from '../models/user';
import serialize from '../serializers/user';

/**
 * Lists all users
 *
 * @param {any} params
 * @param {any} me
 * @return {Promise<any>}
 */
module.exports = (params, me) =>
	new Promise(async (res, rej) => {
		// Get 'limit' parameter
		const [limit, limitErr] = it(params.limit).expect.number().range(1, 100).default(10).qed();
		if (limitErr) return rej('invalid limit param');

		// Get 'since_id' parameter
		const [sinceId, sinceIdErr] = it(params.since_id).expect.id().qed();
		if (sinceIdErr) return rej('invalid since_id param');

		// Get 'max_id' parameter
		const [maxId, maxIdErr] = it(params.max_id).expect.id().qed();
		if (maxIdErr) return rej('invalid max_id param');

		// Check if both of since_id and max_id is specified
		if (sinceId !== null && maxId !== null) {
			return rej('cannot set since_id and max_id');
		}

		// Construct query
		const sort = {
			_id: -1
		};
		const query = {} as any;
		if (sinceId) {
			sort._id = 1;
			query._id = {
				$gt: sinceId
			};
		} else if (maxId) {
			query._id = {
				$lt: maxId
			};
		}

		// Issue query
		const users = await User
			.find(query, {
				limit: limit,
				sort: sort
			});

		// Serialize
		res(await Promise.all(users.map(async user =>
			await serialize(user, me))));
	});
